%let cty = ARG BGR BLZ CAN COL CYP CZE DEU ENG FRA
           GRC HKG HUN IRN ISL ISR ITA KWT LTU LVA
           MAR MDA MKD NLD NOR NZL ROM RUS SCO SGP
           SVK SVN SWE TUR USA SE3;

Options nosource nocenter;

LIBNAME libdat  "c:\pirls\Data" ;
LIBNAME library "c:\pirls\Data" ;

%macro doit;
  %let i = 1;
     %do %while(%length(%scan(&cty,&i)));
       %let ccode = %lowcase(%scan(&cty,&i));

%let fname= ast&ccode.r1 ;

FILENAME rawinp "c:\pirls\Data\&fname..dat" ;

%PUT INFO: data set definition ;
%PUT INFO: data set definitions have to be modified ;

*Output data set ;
data libdat.&fname;

*NOTE: LRECL may need to be modified ;
INFILE rawinp LRECL=1250 END=eof LENGTH=lnline LINE=ln MISSOVER ;

%PUT INFO: shorten variable definitions to save secondary storage area ;
%PUT INFO: refer to the SAS manual for an appropriate length for variables ;

*NOTE: The LENGTH statement in a data step specifies the number of bytes, the
       SAS - sytem uses for storing values of variables in each data set being created. ;
LENGTH DEFAULT=5 ;
LENGTH      IDCNTRY    5 ;
LENGTH      IDSCHOOL   4 ;
LENGTH      IDCLASS    6 ;
LENGTH      IDSTUD     8 ;
LENGTH      IDPOP      3 ;
LENGTH      IDGRADER   3 ;
LENGTH      INPRL01    3 ;
LENGTH      JKZONE     3 ;
LENGTH      JKREP      3 ;
LENGTH      IDTEACH    6 ;
LENGTH      IDLINK     3 ;
LENGTH      NTEACH     3 ;
LENGTH      TCHWGT     8 ;
LENGTH      DPCDATE    6 ;
LENGTH      ASRINF01   8 ;
LENGTH      ASRINF02   8 ;
LENGTH      ASRINF03   8 ;
LENGTH      ASRINF04   8 ;
LENGTH      ASRINF05   8 ;
LENGTH      ASRLIT01   8 ;
LENGTH      ASRLIT02   8 ;
LENGTH      ASRLIT03   8 ;
LENGTH      ASRLIT04   8 ;
LENGTH      ASRLIT05   8 ;
LENGTH      ASRREA01   8 ;
LENGTH      ASRREA02   8 ;
LENGTH      ASRREA03   8 ;
LENGTH      ASRREA04   8 ;
LENGTH      ASRREA05   8 ;


%PUT INFO: Missing category A refers to "not administered" values ;
%PUT INFO: Missing category I refers to "invalid" values ;
%PUT INFO: Missing category B refers to "logically not applicable" values ;
%PUT INFO: Missing category R refers to "not reached" items ;
%PUT INFO: Missing category . refers to missing values ;
MISSING A I B R ;

*NOTE: The INPUT statement reads raw data and assigns variable names.
       The positions specified below correspond to the positions of the variables
       in the raw data file. ;

INPUT
  IDCNTRY       1-   5     
  IDSCHOOL      6-   9     
  IDCLASS      10-  15     
  IDSTUD       16-  23     
  IDPOP        24-  24     
  IDGRADER     25-  25     
  INPRL01      26-  26     
  JKZONE       27-  28     
  JKREP        29-  29     
  IDTEACH      30-  35     
  IDLINK       36-  37     
  NTEACH       38-  39     
  TCHWGT       40-  52 .6
  DPCDATE      53-  58     
  ASRINF01     59-  71 .6
  ASRINF02     72-  84 .6
  ASRINF03     85-  97 .6
  ASRINF04     98- 110 .6
  ASRINF05    111- 123 .6
  ASRLIT01    124- 136 .6
  ASRLIT02    137- 149 .6
  ASRLIT03    150- 162 .6
  ASRLIT04    163- 175 .6
  ASRLIT05    176- 188 .6
  ASRREA01    189- 201 .6
  ASRREA02    202- 214 .6
  ASRREA03    215- 227 .6
  ASRREA04    228- 240 .6
  ASRREA05    241- 253 .6
;

%PUT INFO: assign labels to variables ;
LABEL
  IDCNTRY  = "*COUNTRY ID*"
  IDSCHOOL = "*SCHOOL ID*"
  IDCLASS  = "*CLASS ID*"
  IDSTUD   = "*STUDENT ID*"
  IDPOP    = "*POPULATION ID*"
  IDGRADER = "*GRADE*"
  INPRL01  = "*INPRL01*"
  JKZONE   = "*ZONE CODE*"
  JKREP    = "*REPLICATE CODE*"
  IDTEACH  = "*TEACHER ID*"
  IDLINK   = "*LINK ID*"
  NTEACH   = "*# OF READING TEACHERS COMBINED*"
  TCHWGT   = "*WEIGHT FOR RDG TEACHER DATA COMBINED*"
  DPCDATE  = "DATE OF CREATION AT DPC"
  ASRINF01 = "PLAUSIBLE VALUE: INFORMATION BLOCKS PV1"
  ASRINF02 = "PLAUSIBLE VALUE: INFORMATION BLOCKS PV2"
  ASRINF03 = "PLAUSIBLE VALUE: INFORMATION BLOCKS PV3"
  ASRINF04 = "PLAUSIBLE VALUE: INFORMATION BLOCKS PV4"
  ASRINF05 = "PLAUSIBLE VALUE: INFORMATION BLOCKS PV5"
  ASRLIT01 = "PLAUSIBLE VALUE: LITERARY BLOCKS PV1"
  ASRLIT02 = "PLAUSIBLE VALUE: LITERARY BLOCKS PV2"
  ASRLIT03 = "PLAUSIBLE VALUE: LITERARY BLOCKS PV3"
  ASRLIT04 = "PLAUSIBLE VALUE: LITERARY BLOCKS PV4"
  ASRLIT05 = "PLAUSIBLE VALUE: LITERARY BLOCKS PV5"
  ASRREA01 = "PLAUSIBLE VALUE: ALL BLOCKS PV1"
  ASRREA02 = "PLAUSIBLE VALUE: ALL BLOCKS PV2"
  ASRREA03 = "PLAUSIBLE VALUE: ALL BLOCKS PV3"
  ASRREA04 = "PLAUSIBLE VALUE: ALL BLOCKS PV4"
  ASRREA05 = "PLAUSIBLE VALUE: ALL BLOCKS PV5"
;

%PUT INFO: recode "not administered" data to missing code ".A" ;
*NOTE: If a CHECK variable is coded correctly with a BLANK, it will be recoded
       to "1", otherwise it will be recoded to "0" ;

IF IDCNTRY  = 99998            THEN IDCNTRY  = .A ;
IF IDSCHOOL = 9998             THEN IDSCHOOL = .A ;
IF IDCLASS  = 999998           THEN IDCLASS  = .A ;
IF IDSTUD   = 99999998         THEN IDSTUD   = .A ;
IF IDPOP    = 8                THEN IDPOP    = .A ;
IF IDGRADER = 8                THEN IDGRADER = .A ;
IF INPRL01  = 8                THEN INPRL01  = .A ;
IF JKZONE   = 98               THEN JKZONE   = .A ;
IF JKREP    = 8                THEN JKREP    = .A ;
IF IDTEACH  = 999998           THEN IDTEACH  = .A ;
IF IDLINK   = 98               THEN IDLINK   = .A ;
IF NTEACH   = 98               THEN NTEACH   = .A ;
IF TCHWGT   = 999998           THEN TCHWGT   = .A ;
IF ASRINF01 = 999998           THEN ASRINF01 = .A ;
IF ASRINF02 = 999998           THEN ASRINF02 = .A ;
IF ASRINF03 = 999998           THEN ASRINF03 = .A ;
IF ASRINF04 = 999998           THEN ASRINF04 = .A ;
IF ASRINF05 = 999998           THEN ASRINF05 = .A ;
IF ASRLIT01 = 999998           THEN ASRLIT01 = .A ;
IF ASRLIT02 = 999998           THEN ASRLIT02 = .A ;
IF ASRLIT03 = 999998           THEN ASRLIT03 = .A ;
IF ASRLIT04 = 999998           THEN ASRLIT04 = .A ;
IF ASRLIT05 = 999998           THEN ASRLIT05 = .A ;
IF ASRREA01 = 999998           THEN ASRREA01 = .A ;
IF ASRREA02 = 999998           THEN ASRREA02 = .A ;
IF ASRREA03 = 999998           THEN ASRREA03 = .A ;
IF ASRREA04 = 999998           THEN ASRREA04 = .A ;
IF ASRREA05 = 999998           THEN ASRREA05 = .A ;

%PUT INFO: recode special missing codes to SAS missing codes ;
*NOTE: Special missing codes are recoded to SAS
       character - missing codes;


%PUT INFO: recode omitted data to missing code "." ;
*NOTE: BLANKS are recoded to SYSMIS by SAS ;

IF IDCNTRY  = 99999            THEN IDCNTRY  = . ;
IF IDSCHOOL = 9999             THEN IDSCHOOL = . ;
IF IDCLASS  = 999999           THEN IDCLASS  = . ;
IF IDSTUD   = 99999999         THEN IDSTUD   = . ;
IF IDPOP    = 9                THEN IDPOP    = . ;
IF IDGRADER = 9                THEN IDGRADER = . ;
IF INPRL01  = 9                THEN INPRL01  = . ;
IF JKZONE   = 99               THEN JKZONE   = . ;
IF JKREP    = 9                THEN JKREP    = . ;
IF IDTEACH  = 999999           THEN IDTEACH  = . ;
IF IDLINK   = 99               THEN IDLINK   = . ;
IF NTEACH   = 99               THEN NTEACH   = . ;
IF TCHWGT   = 999999           THEN TCHWGT   = . ;
IF ASRINF01 = 999999           THEN ASRINF01 = . ;
IF ASRINF02 = 999999           THEN ASRINF02 = . ;
IF ASRINF03 = 999999           THEN ASRINF03 = . ;
IF ASRINF04 = 999999           THEN ASRINF04 = . ;
IF ASRINF05 = 999999           THEN ASRINF05 = . ;
IF ASRLIT01 = 999999           THEN ASRLIT01 = . ;
IF ASRLIT02 = 999999           THEN ASRLIT02 = . ;
IF ASRLIT03 = 999999           THEN ASRLIT03 = . ;
IF ASRLIT04 = 999999           THEN ASRLIT04 = . ;
IF ASRLIT05 = 999999           THEN ASRLIT05 = . ;
IF ASRREA01 = 999999           THEN ASRREA01 = . ;
IF ASRREA02 = 999999           THEN ASRREA02 = . ;
IF ASRREA03 = 999999           THEN ASRREA03 = . ;
IF ASRREA04 = 999999           THEN ASRREA04 = . ;
IF ASRREA05 = 999999           THEN ASRREA05 = . ;

%PUT INFO: assign variable labels to recoded variables ;
LABEL
;

proc sort ; by idcntry idteach idlink;

       %let i = %eval(&i + 1);
     %end;
run;
%mend doit;

%doit;









